@@ -4,12 +4,13 @@ from abc import ABC, abstractmethod
 from typing import Union, List, Dict
 
 from . import api
+from ._types import MessageTypes, ChannelTypes, SlowModeTypes, MessageFlagModes
 from .gateway import Requestable, Gateway
 from .interface import LazyLoadable
 from .permission import ChannelPermission, PermissionHolder
-from ._types import MessageTypes, ChannelTypes, SlowModeTypes
-from .user import User
-from .util import unpack_value
+from .role import Role
+from .user import User, GuildUser
+from .util import unpack_value, unpack_id
 
 
 class Channel(LazyLoadable, Requestable, ABC):
@@ -84,6 +85,49 @@ class PublicChannel(Channel, PermissionHolder, ABC):
         await self.load()
         return rt
 
+    async def list_users(self,
+                         search: str = None,
+                         role: Union[Role, str, int] = None,
+                         mobile_verified: bool = None,
+                         active_time: int = None,
+                         joined_at: int = None,
+                         page: int = 1,
+                         page_size: int = 50,
+                         filter_user_id: str = None) -> List[User]:
+        """list the users who can see this channel"""
+        params = {'guild_id': self.guild_id, 'channel_id': self.id, 'page': page, 'page_size': page_size}
+        if search is not None:
+            params['search'] = search
+        if role is not None:
+            params['role_id'] = unpack_id(role)
+        if mobile_verified is not None:
+            params['mobile'] = 1 if mobile_verified else 0
+        if active_time is not None and active_time in [0, 1]:
+            params['active_time'] = active_time
+        if joined_at is not None and joined_at in [0, 1]:
+            params['joined_at'] = joined_at
+        if filter_user_id is not None:
+            params['filter_user_id'] = filter_user_id
+        users = await self.gate.exec_paged_req(api.Guild.userList(**params))
+        return [User(_gate_=self.gate, _lazy_loaded_=True, **i) for i in users]
+
+    async def list_messages(self,
+                            page_size: int = None,
+                            pin: int = None,
+                            flag: MessageFlagModes = None,
+                            msg_id: str = None) -> Dict:
+        """list the messages in this channel (only for public channel now)"""
+        params = {'target_id': self.id}
+        if page_size is not None:
+            params['page_size'] = page_size
+        if pin is not None:
+            params['pin'] = pin
+        if flag is not None:
+            params['flag'] = flag
+        if msg_id is not None:
+            params['msg_id'] = msg_id
+        return await self.gate.exec_req(api.Message.list(**params))
+
 
 class PublicTextChannel(PublicChannel):
     """
@@ -134,8 +178,13 @@ class PublicVoiceChannel(PublicChannel):
         user_ids = [u.id if isinstance(u, User) else u for u in users]
         return await self.gate.exec_req(api.Channel.moveUser(target_id=self.id, user_ids=user_ids))
 
+    async def fetch_user_list(self) -> List[GuildUser]:
+        """get users chatting in the voice channel"""
+        users = await self.gate.exec_paged_req(api.Channel.userList(channel_id=self.id))
+        return [GuildUser(_gate_=self.gate, guild_id=self.guild_id, _lazy_loaded_=True, **i) for i in users]
 
-def public_channel_factory(_gate_: Gateway, **kwargs) -> PublicChannel:
+
+def public_channel_factory(_gate_: Gateway, **kwargs) -> Union[PublicTextChannel, PublicVoiceChannel]:
     """factory function to build a channel object"""
     kwargs['type'] = kwargs['type'] if isinstance(kwargs['type'], ChannelTypes) else ChannelTypes(kwargs['type'])
     if kwargs['type'] == ChannelTypes.TEXT:
