@@ -5,51 +5,18 @@ import warnings
 from typing import List, Union, Dict, IO
 
 from . import api
-from .channel import Channel, public_channel_factory, PublicChannel, PublicVoiceChannel
-from .gateway import Requestable, Gateway
+from .channel import Channel, public_channel_factory, PublicChannel, PublicVoiceChannel, PublicTextChannel
+from .gateway import Requestable
 from .interface import LazyLoadable
 from .permission import PermissionHolder, ChannelPermission
 from .role import Role
-from ._types import ChannelTypes, GuildMuteTypes
-from .user import User
-from .util import unpack_id
+from ._types import ChannelTypes, GuildMuteTypes, BadgeTypes
+from .user import User, GuildUser
+from .util import unpack_id, unpack_value
 
 log = logging.getLogger(__name__)
 
 
-class GuildUser(User):
-    """a user in guild
-
-    with some fields more than User"""
-    guild_id: str
-    joined_at: int
-    active_time: int
-    roles: List[int]
-    gate: Gateway
-
-    def __init__(self, **kwargs):
-        self.roles = kwargs.get('roles', [])
-        self.guild_id = kwargs.get('guild_id', '')
-        self.joined_at = kwargs.get('joined_at', 0)
-        self.active_time = kwargs.get('active_time', 0)
-        super().__init__(**kwargs)
-
-    async def fetch_roles(self, **kwargs) -> List[Role]:
-        """
-        Get the user roles in this guild
-
-        paged req, support standard pagination args
-
-        :return: A list for Role
-        """
-        guild_roles = (await self.gate.exec_paged_req(api.GuildRole.list(self.guild_id), **kwargs))
-        rt: List[Role] = []
-        for role in guild_roles:
-            if role['role_id'] in self.roles:
-                rt.append(Role(**role))
-        return rt
-
-
 class GuildBoost:
     """Guild boost"""
     user_id: str
@@ -66,6 +33,31 @@ class GuildBoost:
         self.user = User(**kwargs.get('user'), _gate_=kwargs.get('_gate_', None))
 
 
+class GuildEmoji(Requestable):
+    """
+    The custom emojis in the guild
+    """
+    guild_id: str
+    name: str
+    id: str
+    user: GuildUser
+
+    def __init__(self, **kwargs):
+        self.gate = kwargs.get('_gate_', None)
+        self.guild_id = kwargs.get("guild_id")
+        self.name = kwargs.get("name")
+        self.id = kwargs.get("id")
+        self.user = GuildUser(_gate_=self.gate, guild_id=self.guild_id, **kwargs.get("user_info"))
+
+    async def update(self, name: str):
+        """update this emoji's name"""
+        await self.gate.exec_req(api.GuildEmoji.update(id=self.id, name=name))
+
+    async def delete(self):
+        """delete this custom emoji"""
+        await self.gate.exec_req(api.GuildEmoji.delete(id=self.id))
+
+
 class ChannelCategory(PermissionHolder, Requestable):
     """represent a channel set"""
 
@@ -103,17 +95,24 @@ class ChannelCategory(PermissionHolder, Requestable):
         """pop a channel(default last) from this category"""
         return self._channels.pop(index)
 
-    async def create_channel(self,
-                             name: str,
-                             type: ChannelTypes = None,
-                             limit_amount: int = None,
-                             voice_quality: int = None) -> PublicChannel:
+    async def create_text_channel(self,
+                                  name: str) -> PublicTextChannel:
+        """create a text channel in this channel category
+
+        docs: https://developer.kaiheila.cn/doc/http/channel#%E5%88%9B%E5%BB%BA%E9%A2%91%E9%81%93"""
+        params = {'name': name, 'guild_id': self.guild_id, 'parent_id': self.id, 'type': ChannelTypes.TEXT.value}
+        pc = public_channel_factory(self.gate, **(await self.gate.exec_req(api.Channel.create(**params))))
+        self._channels.append(pc)
+        return pc
+
+    async def create_voice_channel(self,
+                                   name: str,
+                                   limit_amount: int = None,
+                                   voice_quality: int = None) -> PublicVoiceChannel:
         """create a channel in this channel category
 
         docs: https://developer.kaiheila.cn/doc/http/channel#%E5%88%9B%E5%BB%BA%E9%A2%91%E9%81%93"""
-        params = {'name': name, 'guild_id': self.guild_id, 'parent_id': self.id}
-        if type is not None:
-            params['type'] = type.value
+        params = {'name': name, 'guild_id': self.guild_id, 'parent_id': self.id, 'type': ChannelTypes.VOICE.value}
         if limit_amount:
             params['limit_amount'] = limit_amount
         if voice_quality:
@@ -266,9 +265,9 @@ class Guild(LazyLoadable, Requestable):
         user = await self.gate.exec_req(api.User.view(user_id=user_id, guild_id=self.id))
         return GuildUser(guild_id=self.id, _gate_=self.gate, _lazy_loaded_=True, **user)
 
-    async def set_user_nickname(self, user: Union[User, str], new_nickname: str):
+    async def set_user_nickname(self, user: Union[User, str], nickname: str):
         """set the user's nickname in this guild"""
-        await self.gate.exec_req(api.Guild.nickname(guild_id=self.id, nickname=new_nickname, user_id=unpack_id(user)))
+        await self.gate.exec_req(api.Guild.nickname(guild_id=self.id, nickname=nickname, user_id=unpack_id(user)))
 
     async def fetch_roles(self, force_update: bool = True) -> List[Role]:
         """fetch the role list in the guild"""
@@ -307,21 +306,25 @@ class Guild(LazyLoadable, Requestable):
         return await self.gate.exec_req(
             api.GuildRole.revoke(guild_id=self.id, user_id=unpack_id(user), role_id=unpack_id(role)))
 
-    async def create_channel(self,
-                             name: str,
-                             type: ChannelTypes = None,
-                             category: Union[str, ChannelCategory] = None,
-                             limit_amount: int = None,
-                             voice_quality: int = None):
-        """create a channel in the guild
+    async def create_text_channel(self,
+                                  name: str,
+                                  category: Union[str, ChannelCategory] = None) -> PublicTextChannel:
+        """create a text channel in the guild
 
         docs: https://developer.kaiheila.cn/doc/http/channel#%E5%88%9B%E5%BB%BA%E9%A2%91%E9%81%93"""
-        params = {'name': name, 'guild_id': self.id}
-        if type is not None:
-            if type == ChannelTypes.CATEGORY:
-                params['is_category'] = 1
-            else:
-                params['type'] = type.value
+        params = {'name': name, 'guild_id': self.id, 'type': ChannelTypes.TEXT.value}
+        if category:
+            params['parent_id'] = unpack_id(category)
+        return public_channel_factory(self.gate, **(await self.gate.exec_req(api.Channel.create(**params))))
+
+    async def create_voice_channel(self, name: str,
+                                   category: Union[str, ChannelCategory] = None,
+                                   limit_amount: int = None,
+                                   voice_quality: int = None) -> PublicVoiceChannel:
+        """create a voice channel in the guild
+
+        docs: https://developer.kaiheila.cn/doc/http/channel#%E5%88%9B%E5%BB%BA%E9%A2%91%E9%81%93"""
+        params = {'name': name, 'guild_id': self.id, 'type': ChannelTypes.VOICE.value}
         if category:
             params['parent_id'] = unpack_id(category)
         if limit_amount:
@@ -330,6 +333,14 @@ class Guild(LazyLoadable, Requestable):
             params['voice_quality'] = voice_quality
         return public_channel_factory(self.gate, **(await self.gate.exec_req(api.Channel.create(**params))))
 
+    async def create_channel_category(self, name: str) -> ChannelCategory:
+        """create a channel category in the guild
+
+        docs: https://developer.kaiheila.cn/doc/http/channel#%E5%88%9B%E5%BB%BA%E9%A2%91%E9%81%93"""
+        params = {'guild_id': self.id, 'name': name, 'is_category': 1}
+        return ChannelCategory(_gate_=self.gate, **(
+            await self.gate.exec_req(api.Channel.create(**params))))
+
     async def delete_channel(self, channel: Union[Channel, str]):
         """delete the channel from the guild"""
         return await self.gate.exec_req(api.Channel.delete(unpack_id(channel)))
@@ -366,14 +377,15 @@ class Guild(LazyLoadable, Requestable):
         user_id = user.id if isinstance(user, User) else user
         return await self.gate.exec_req(api.GuildMute.delete(guild_id=self.id, user_id=user_id, type=type.value))
 
-    async def fetch_emoji_list(self) -> List[Dict]:
+    async def fetch_emoji_list(self) -> List[GuildEmoji]:
         """fetch guild emoji list
 
         :returns: a list of emoji dict, dict contains {'name', 'id', 'user_info': who uploaded the emoji}
         """
-        return await self.gate.exec_paged_req(api.GuildEmoji.list(guild_id=self.id))
+        emojis = await self.gate.exec_paged_req(api.GuildEmoji.list(guild_id=self.id))
+        return [GuildEmoji(_gate_=self.gate, guild_id=self.id, **i) for i in emojis]
 
-    async def create_emoji(self, emoji: Union[IO, str], *, name: str = None) -> Dict:
+    async def create_emoji(self, emoji: Union[IO, str], *, name: str = None) -> GuildEmoji:
         """upload a custom emoji to the guild
 
         :returns: an emoji dict. For emoji dict structure, please refer to fetch_emoji_list() doc
@@ -383,18 +395,19 @@ class Guild(LazyLoadable, Requestable):
         params = {'guild_id': self.id, 'emoji': emoji}
         if name is not None:
             params['name'] = name
-        return await self.gate.exec_req(api.GuildEmoji.create(**params))
+        emoji_info = await self.gate.exec_req(api.GuildEmoji.create(**params))
+        return GuildEmoji(_gate_=self.gate, guild_id=self.id, **emoji_info)
 
-    async def update_emoji(self, id: str, *, name: str = None):
+    async def update_emoji(self, emoji: Union[GuildEmoji, str], *, name: str = None):
         """update a custom emoji's name"""
-        params = {'id': id}
+        params = {'id': unpack_id(emoji)}
         if name is not None:
             params['name'] = name
         return await self.gate.exec_req(api.GuildEmoji.update(**params))
 
-    async def delete_emoji(self, id: str):
+    async def delete_emoji(self, emoji: Union[GuildEmoji, str]):
         """delete a custom emoji"""
-        return await self.gate.exec_req(api.GuildEmoji.delete(id))
+        return await self.gate.exec_req(api.GuildEmoji.delete(unpack_id(emoji)))
 
     async def fetch_boost(self, start_time: int = 0, end_time: int = int(time.time()), **kwargs) -> List[GuildBoost]:
         """
@@ -406,3 +419,7 @@ class Guild(LazyLoadable, Requestable):
         boost_list = await self.gate.exec_paged_req(
             api.GuildBoost.history(guild_id=self.id, start_time=start_time, end_time=end_time), **kwargs)
         return [GuildBoost(**item, _gate_=self.gate) for item in boost_list]
+
+    async def fetch_badge(self, style: Union[int, BadgeTypes] = BadgeTypes.NAME) -> bytes:
+        """get the badge of the guild"""
+        return await self.gate.exec_req(api.Badge.guild(guild_id=self.id, style=unpack_value(style)))
