@@ -67,29 +67,44 @@ class WebsocketReceiver(Receiver):
             except ConnectionResetError:
                 return
             except Exception as e:
-                log.exception('error raised during websocket heartbeat', exc_info=e)
+                log.exception('error raised during websocket heartbeat',
+                              exc_info=e)
+
+    async def _get_gateway(self, cs: ClientSession):
+        headers = {
+            'Authorization': f'Bot {self._cert.token}',
+            'Content-type': 'application/json'
+        }
+        params = {'compress': 1 if self.compress else 0}
+        async with cs.get(f"{API}/gateway/index",
+                          headers=headers,
+                          params=params) as res:
+            res_json = await res.json()
+            if res_json['code'] != 0:
+                log.error(f'getting gateway: {res_json}')
+                return
+
+            self._RAW_GATEWAY = res_json['data']['url']
+
+    async def _connect_gateway_and_handle_msg(self, cs: ClientSession):
+        async with cs.ws_connect(self._RAW_GATEWAY) as ws_conn:
+            asyncio.ensure_future(self.heartbeat(ws_conn), loop=self.loop)
+
+            log.info('[ init ] launched')
+            try:
+                async for raw in ws_conn:
+                    raw: WSMessage
+                    await self._handle_raw(raw)
+            except Exception:
+                log.exception(
+                    'error raised during websocket receive, reconnect automatically'
+                )
 
     async def start(self):
         async with ClientSession(loop=self.loop) as cs:
-            headers = {'Authorization': f'Bot {self._cert.token}', 'Content-type': 'application/json'}
-            params = {'compress': 1 if self.compress else 0}
             while True:
-                async with cs.get(f"{API}/gateway/index", headers=headers, params=params) as res:
-                    res_json = await res.json()
-                    if res_json['code'] != 0:
-                        log.error(f'getting gateway: {res_json}')
-                        return
-
-                    self._RAW_GATEWAY = res_json['data']['url']
-
-                async with cs.ws_connect(self._RAW_GATEWAY) as ws_conn:
-                    asyncio.ensure_future(self.heartbeat(ws_conn), loop=self.loop)
-
-                    log.info('[ init ] launched')
-
-                    async for raw in ws_conn:
-                        raw: WSMessage
-                        await self._handle_raw(raw)
+                await self._get_gateway(cs)
+                await self._connect_gateway_and_handle_msg(cs)
 
     async def _handle_raw(self, raw: WSMessage):
         try:
@@ -147,7 +162,8 @@ class WebhookReceiver(Receiver):
             if not pkg:  # empty pkg
                 return web.Response()
 
-            if pkg['d']['verify_token'] != self._cert.verify_token:  # check verify_token
+            if pkg['d'][
+                    'verify_token'] != self._cert.verify_token:  # check verify_token
                 return web.Response()
 
             if self._is_dup(pkg):  # dup pkg
@@ -155,7 +171,8 @@ class WebhookReceiver(Receiver):
 
             if pkg['s'] == 0:
                 pkg = pkg['d']
-                if pkg['type'] == 255 and pkg['channel_type'] == 'WEBHOOK_CHALLENGE':
+                if pkg['type'] == 255 and pkg[
+                        'channel_type'] == 'WEBHOOK_CHALLENGE':
                     return web.json_response({'challenge': pkg['challenge']})
                 await self.pkg_queue.put(pkg)
 
